=begin

= rd2latex-lib.rb

Copyright (C) 2001  Shugo Maeda <shugo@ruby-lang.org>

This library is distributed under the terms of the Ruby license.
You can freely distribute/modify this library.

=end

require "rd/rdvisitor"
require "rd/version"

module RD
  class RD2LaTeXVisitor < RDVisitor
    include AutoLabel

    SYSTEM_NAME = "RDtool -- RD2LaTeXVisitor"
    SYSTEM_VERSION = "0.1"
    VERSION = Version.new_from_version_string(SYSTEM_NAME, SYSTEM_VERSION)

    def self.version
      VERSION
    end

    OUTPUT_SUFFIX = "tex"
    INCLUDE_SUFFIX = ["tex"]

    METACHAR = { 
      '#'  => '\#',
      "\\" => '\\\\',
      "$"  => '\$',
      '_'  => '\_',
      '&'  => '\&',
      '<'  => '$<$',
      '>'  => '$>$',
      '|'  => '$|$' #"
    }

    attr_accessor :documentclass, :documentclass_option,
      :title, :author, :date, :preamble_file,
      :maketitle, :maketoc

    def initialize
      @index = {}
      @documentclass = "jarticle"
      @documentclass_option = nil
      @title = nil
      @author = nil
      @date = nil
      @preamble_file = nil
      @maketitle = false
      @maketoc = false
      super
    end

    def visit(tree)
      prepare_labels(tree, "label:")
      super(tree)
    end

    def apply_to_DocumentElement(element, content)
      if @documentclass_option
	option = "[#{@documentclass_option}]"
      else
	option = ""
      end
      str = %Q(\\documentclass#{option}{#{@documentclass}}\n)
      if @title
	str << %Q(\\title{#{@title}}\n)
      end
      if @author
	str << %Q(\\author{#{@author}}\n)
      end
      if @date
	str << %Q(\\date{#{@date}}\n)
      end
      if @preamble_file
	str << open(@preamble_file).read
      end
      str << "\\begin{document}\n"
      if @maketitle
	str << "\\maketitle\n"
      end
      if @maketoc
	str << "\\tableofcontents\n\\clearpage\n"
      end
      str << content.join
      str << "\n"
      str << %Q[\\end{document}\n]
      return str
    end

    def apply_to_Headline(element, title)
      anchor = get_anchor(element)
      if element.level < 4
	%Q[\\#{"sub"*(element.level-1)}section{\\label{#{anchor}} #{title}}\n\n]
      else
	%Q[\\paragraph{\\label{#{anchor}} #{title}}\n\n]
      end
    end

    # RDVisitor#apply_to_Include 

    def apply_to_TextBlock(element, content)
      content.join + "\n"
    end

    def apply_to_Verbatim(element)
      content = []
      element.each_line do |i|
	content.push(i)
      end
      %Q[\\begin{verbatim}\n#{content.join}\\end{verbatim}\n]
    end

    def apply_to_ItemList(element, items)
      %Q[\\begin{itemize}\n#{items.join("\n")}\n\\end{itemize}\n]
    end

    def apply_to_EnumList(element, items)
      %Q[\\begin{enumerate}\n#{items.join("\n")}\n\\end{enumerate}\n]
    end

    def apply_to_DescList(element, items)
      %Q[\\begin{description}\n#{items.join("\n")}\n\\end{description}\n]
    end

    def apply_to_ItemListItem(element, content)
      content = content.join.sub(/\n*\z/, "")
      %Q[\\item #{content}]
    end

    def apply_to_EnumListItem(element, content)
      content = content.join.sub(/\n*\z/, "")
      %Q[\\item #{content}]
    end

    def apply_to_DescListItem(element, term, description)
      term = term.join.chomp
      term.gsub!(/[{}]/, "\\\\\\&")
      term.gsub!(/\[.*?\]/, "{\\&}")
      anchor = get_anchor(element.term)
      if description.empty?
	%Q(\\item[\\label{#{anchor}} #{term}] \\quad)
      else
	description = description.join.sub(/\n*\z/, "")
        %Q(\\item[\\label{#{anchor}} #{term}] \\quad \\\\\n) +
	  %Q[#{description}]
      end
    end

    def apply_to_StringElement(element)
      apply_to_String(element.content)
    end

    def apply_to_Emphasis(element, content)
      %Q[{\\em #{content.join}}]
    end

    def apply_to_Code(element, content)
      c = element.content.join
      sep = %w(! @ # $ % & * - _ + | ).find{|s| c !~ /#{s}/}
      sep or raise "no candidate for verb paren"
      %Q[\\verb#{sep}#{c}#{sep}]
    end

    def apply_to_Var(element, content)
      %Q[{\\tt #{content.join}}]
    end

    def apply_to_Keyboard(element, content)
      %Q[{\\sf #{content.join}}]
    end

    def apply_to_Index(element, content)
      tmp = []
      element.each do |i|
	tmp.push(i) if i.is_a?(String)
      end
      key = meta_char_escape(tmp.join)
      if @index.has_key?(key)
	# warning?
      else
	num = @index[key] = @index.size
	#%Q[<A NAME="index:#{num}">#{content.join("")}</A>]
      end
    end

    def apply_to_Reference(element, content)
      case element.label
      when Reference::URL
	apply_to_RefToURL(element, content)
      when Reference::RDLabel
	if element.label.filename
	  apply_to_RefToOtherFile(element, content)
	else
	  apply_to_RefToElement(element, content)
	end
      end
    end

    def apply_to_RefToElement(element, content)
      if anchor = refer(element)
	%Q[\\ref{#{anchor}}]
      else
	content.join
      end
    end

    def apply_to_RefToOtherFile(element, content)
      content.join
    end

    def apply_to_RefToURL(element, content)
      content.join
    end

    def apply_to_Footnote(element, content)
      %Q[{\\footnote #{content.join}}]
    end

    def apply_to_Verb(element)
      c = element.content.join
      sep = %w(! @ $ % / * - _ + | ).find{|s| c !~ /#{s}/}
      sep or raise "no candidate for verb paren"
      %Q[\\verb#{sep}#{c}#{sep}]
    end

    def apply_to_String(element)
      meta_char_escape(element)
    end

    private

    def meta_char_escape(str)
      str.gsub(/[#\\$_&<>|]/) {
	METACHAR[$&]
      }
    end
  end
end

$Visitor_Class = RD::RD2LaTeXVisitor
$RD2_Sub_OptionParser = "rd/rd2latex-opt"
